% $Id: Configuration.oz,v 1.9 2009/01/11 19:50:34 leavens Exp leavens $
% Configurations (States)
% AUTHOR: Gary T. Leavens

% Representation: 
% <State> ::= config(<Stack> <Store>) | msg(<String>)
% <Stack> ::= <List <Stmt>#<Environment>>

\insert 'Store.oz'
\insert 'Unparse.oz'
declare

fun {InputToConfig Stmt}
   %% ENSURES: Result is an initial configuration containing Stmt
   %% where the initial environment and store are empty
   {MakeConfig Stmt {InitEnv} {EmptyStore}}
end

fun {MakeConfig Stmt Env Store}
   %% ENSURES: Result is an initial configuration containing Stmt
   %% and the given environment and store
   config(Stmt#Env|nil Store)
end

fun {IsTerminalConfig Config}
   %% ENSURES: Result is true when the configuration is terminal
   case Config of
      config(nil _) then true
   [] config(_ _) then false
   else true
   end
end

fun {Output Config}
   %% REQUIRES: Config is terminal
   %% ENSURES: Result is the output of Config (a store or message)
   case Config of
      config(nil Store) then Store
   [] msg(Message) then Message
   else raise cantOutputFromNonTerminalConfig end
   end
end

fun {StoreOfConfig config(_ Store)}
   %% ENSURES: Result is the store of a normal config
   Store
end

fun {StackOfConfig config(Stack _)}
   %% ENSURES: Result is the stack from a normal config
   Stack
end

local
   fun {EqualStack Stack1 Stack2}
      %% ENSURES: Result is true if Stack1 is equivalent to Stack2
      case Stack1#Stack2 of
         nil#nil then true
      [] (Stmt1#Env1|Rest1)#(Stmt2#Env2|Rest2) then
         Stmt1 == Stmt2 andthen {EqualEnv Env1 Env2}
         andthen {EqualStack Rest1 Rest2}
      else false
      end
   end
in
   fun {EqualConfig config(Stack1 Store1) config(Stack2 Store2)}
      %% ENSURES: Result is true if the two configurations are equal
      {EqualStore Store1 Store2} andthen {EqualStack Stack1 Stack2}
   end
end

fun {StringForStack Stack}
   %% ENSURES: Result is a virtual string representing the Stack argument
   case Stack of
      nil then 'n'#'il'
   [] (Stmt#Env)|Rest then
      '('#{Unparse Stmt}#', '#{StringForEnv Env}#')'
      # '|' # {StringForStack Rest}
   end
end

fun {StringForConfig C}
   %% ENSURES: Result is a virtual string representing C
   case C of
      config(Stack Store) then
      '(' # {StringForStack Stack} # ', ' # {StringForStore Store} # ')'
   [] msg(Message) then Message
   end
end
